@@ -4,11 +4,12 @@
 __author__ = 'Frederic Escudie'
 __copyright__ = 'Copyright (C) 2018 IUCT-O'
 __license__ = 'GNU General Public License'
-__version__ = '1.4.0'
+__version__ = '1.7.0'
 __email__ = 'escudie.frederic@iuct-oncopole.fr'
 __status__ = 'prod'
 
 from anacore.region import RegionTree, RegionList
+from anacore.sequenceIO import getStrandedSeqFromPos
 
 
 class Gene(RegionTree):
@@ -80,24 +81,29 @@ class Transcript(RegionTree):
         :rtype: genomicRegion.Transcript
         """
         RegionTree.__init__(self, start, end, strand, reference, name, annot, parent, children)
-        self.proteins = RegionList()
-        if proteins is not None:
-            self.setProteins(proteins)
+        self._proteins = RegionList()
+        self.proteins = proteins
 
-    def setProteins(self, proteins):
+    @property
+    def proteins(self):
+        return self._proteins
+
+    @proteins.setter
+    def proteins(self, proteins):
         """
         Change proteins linked with transcript. Before the add of the new proteins the old are unlinked (prot.transcript is set to None).
 
         :param proteins: The new linked proteins.
         :type proteins: list
         """
-        # Remove previous not in new proteins
+        # Remove previous proteins
         for idx_prot, prot in enumerate(self.proteins):
-            prot.transcript = None
-        self.proteins = RegionList()
+            prot.transcript = None  # Remove link in protein
+        self._proteins = RegionList()  # Remove link in transcript
         # Add new proteins
-        for curr_prot in proteins:
-            curr_prot.setTranscript(self)
+        if proteins is not None:
+            for curr_prot in proteins:
+                curr_prot.transcript = self
 
     def addProtein(self, protein):
         """
@@ -106,7 +112,16 @@ class Transcript(RegionTree):
         :param protein: The protein to add.
         :type protein: genomicRegion.Protein
         """
-        protein.setTranscript(self)
+        protein.transcript = self
+
+    def delProtein(self, protein):
+        """
+        Remove link with a protein.
+
+        :param protein: The protein to remove.
+        :type protein: genomicRegion.Protein
+        """
+        protein.transcript = None
 
     def length(self):
         """
@@ -141,16 +156,45 @@ class Transcript(RegionTree):
         :return: Coordinate on region (1-based).
         :rtype: int
         """
-        raise NotImplementedError
+        if chr_pos < self.start or chr_pos > self.end:
+            raise ValueError("The position {} is out of transcript {}.".format(chr_pos, self))
+        if self.strand is None:
+            raise Exception("Cannot return region position from reference position because the strand is None ({}).".format(
+                self
+            ))
+        transcript_pos = 0
+        idx = 0
+        found = False
+        if self.strand == "+":
+            while not found:
+                curr_exon = self.children[idx]
+                if chr_pos > curr_exon.end:  # Next exon
+                    transcript_pos += curr_exon.length()
+                    idx += 1
+                elif chr_pos >= curr_exon.start:  # In exon
+                    transcript_pos += chr_pos - curr_exon.start + 1
+                    found = True
+                else:  # In previous intron
+                    raise Exception("Position {}:{} is in intron of {}.".format(self.reference.name, chr_pos, self))
+        else:  # Strand -
+            while not found:
+                curr_exon = self.children[idx]
+                if chr_pos < curr_exon.start:  # Next exon
+                    transcript_pos += curr_exon.length()
+                    idx += 1
+                elif chr_pos <= curr_exon.end:  # In exon
+                    transcript_pos += curr_exon.end - chr_pos + 1
+                    found = True
+                else:  # In previous intron
+                    raise Exception("Position {}:{} is in intron of {}.".format(self.reference.name, chr_pos, self))
+        return transcript_pos
 
     def getSubFromRegionPos(self, transcript_pos):
         """
-        Return exon and coordinate on his from the coordinate on transcript.
+        Return exon and coordinate on its from coordinate on transcript. Coordinate on exon is stranded:
 
-        Coordinate on exon is stranded:
-
-        - pos 2 in region test:10-20(+) correspond to ref position 12
-        - pos 2 in region test:10-20(-) correspond to ref position 18
+            * pos 2 in region test:10-20(+) correspond to ref position 12
+            * pos 2 in region test:10-20(-) correspond to ref position 18
 
         :param transcript_pos: The coordinate on transcript (1-based).
         :type transcript_pos: int
@@ -245,20 +289,29 @@ class Protein(RegionTree):
         :rtype: genomicRegion.Protein
         """
         RegionTree.__init__(self, start, end, strand, reference, name, annot, parent, children)
-        self.setTranscript(transcript)
+        self._transcript = None
+        self.transcript = transcript
 
-    def setTranscript(self, transcript):
+    @property
+    def transcript(self):
+        return self._transcript
+
+    @transcript.setter
+    def transcript(self, transcript):
         """
         Add link between self and his transcript.
 
         :param transcript: The transcript to link.
         :type transcript: genomicRegion.Transcript
         """
-        self.transcript = transcript
+        self._transcript = transcript
+        # Manage link in transcript
         if transcript is not None:
+            # Add link in transcript
             if self not in transcript.proteins:
                 transcript.proteins.append(self)
-        else:
+        else:  # New transcript is None
+            # Remove link in previous transcript
             prot_idx = None
             try:
                 prot_idx = transcript.proteins.index(self)
@@ -275,7 +328,7 @@ class Protein(RegionTree):
         :rtype: int
         """
         length = 0
-        for curr_cds in self.children:
+        for curr_cds in self._getCDS():
             length += curr_cds.length()
         return length
 
@@ -350,7 +403,7 @@ class Protein(RegionTree):
                 self
             ))
         chained_cds_pos = None
-        cds = self.children
+        cds = self._getCDS()
         cds_idx = 0
         if cds[cds_idx].strand == "+":
             if chr_pos <= cds[-1].end and chr_pos >= cds[0].start:
@@ -370,10 +423,10 @@ class Protein(RegionTree):
 
     def getSubFromRegionPos(self, protein_pos, codon_pos):
         """
-        Return CDS and coordinate on his from the coordinate on protein.
-        The coordinate on CDS is stranded:
-            With region test:10-20(+) the pos 2 correspond to ref position 12
-            With region test:10-20(-) the pos 2 correspond to ref position 18
+        Return CDS and coordinate on its from coordinate on protein. The coordinate on CDS is stranded:
+
+            * With region test:10-20(+) the pos 2 correspond to ref position 12
+            * With region test:10-20(-) the pos 2 correspond to ref position 18
 
         :param protein_pos: Coordinate of the amino acid on protein (1-based).
         :type protein_pos: int.
@@ -383,13 +436,14 @@ class Protein(RegionTree):
         :rtype: genomicRegion.CDS, int
         """
         chained_cds_pos = self.getNtPosFromRegionPos(protein_pos, codon_pos)
+        cds = self._getCDS()
         walk_pos = 0
         region_idx = 0
         child = None
         while child is None:
-            curr_child_length = self.children[region_idx].length()
+            curr_child_length = cds[region_idx].length()
             if walk_pos <= chained_cds_pos and (walk_pos + curr_child_length) >= chained_cds_pos:
-                child = self.children[region_idx]
+                child = cds[region_idx]
             else:
                 walk_pos += curr_child_length
                 region_idx += 1
@@ -440,9 +494,22 @@ class Protein(RegionTree):
         :return: True if the region contains the evaluated region.
         :rtype: bool
         """
-        cds = self.children if len(self.children) > 0 else self.getCDSFromTranscript()
+        cds = self._getCDS()
         return len(cds.getContainers(eval_region)) > 0
 
+    def _getCDS(self):
+        """
+        Init self.children if necessary and return the list of CDS.
+
+        :param eval_region: The evaluated region.
+        :type eval_region: Region
+        :return: The list of CDS of the protein in protein strand order.
+        :rtype: anacore.region.Regionlist
+        """
+        if len(self.children) == 0:
+            self.children = self.getCDSFromTranscript()
+        return self.children
+
     def hasOverlap(self, eval_region):
         """
         Return True if at least one CDS has an overlap with eval_region.
@@ -454,8 +521,69 @@ class Protein(RegionTree):
         """
         has_overlap = False
         if self.reference.name == eval_region.reference.name:
-            cds = self.children if len(self.children) > 0 else self.getCDSFromTranscript()
+            cds = self._getCDS()
             for curr_cds in cds:
                 if not curr_cds.start > eval_region.end and not curr_cds.end < eval_region.start:
                     has_overlap = True
         return has_overlap
+
+    def getCodonRefPos(self, aa_pos):
+        """
+        Return stranded positions on reference for the nt corresponding to the amino acid at provided position on protein.
+
+        :param aa_pos: Position of the amino acid in the protein.
+        :type aa_pos: int
+        :return: Stranded positions on reference for the nt corresponding to the amino acid at provided position on protein.
+        :rtype: list
+        """
+        codon_pos_on_ref = []
+        for curr_pos in range(1, 4):
+            codon_pos_on_ref.append(self.getPosOnRef(aa_pos, curr_pos))
+        return codon_pos_on_ref
+
+    def getCodonSeqFromProtPos(self, aa_pos, sequence_reader):
+        """
+        Return the codon nucleotids corresponding to the amino acid at the provided position on protein.
+
+        :param aa_pos: Position of the amino acid in the protein.
+        :type aa_pos: int
+        :param sequence_reader: File handle to the reference sequences file.
+        :type sequence_reader: anacore.sequenceIO.IdxFastaIO
+        :return: Codon nucleotids corresponding to the amino acid at the provided position on protein.
+        :rtype: str
+        """
+        codon_pos_on_ref = self.getCodonRefPos(aa_pos)
+        return getStrandedSeqFromPos(
+            self.reference.name,
+            codon_pos_on_ref,
+            self.strand,
+            sequence_reader
+        )
+
+    def getCodonInfo(self, ref_pos, sequence_reader):
+        """
+        Return the amino acid position, position in codon and codon sequence for the position on reference.
+
+        :param ref_pos: Position on the reference.
+        :type ref_pos: int
+        :param sequence_reader: File handle to the reference sequences file.
+        :type sequence_reader: anacore.sequenceIO.IdxFastaIO
+        :return: Amino acid position, position in codon and codon sequence for the position on reference.
+        :rtype: (int, int, str)
+        """
+        # Get codon pos
+        aa_pos, pos_in_codon = self.getPosOnRegion(ref_pos)
+        codon_pos_on_ref = []
+        for curr_pos in range(1, 4):
+            if curr_pos == pos_in_codon:
+                codon_pos_on_ref.append(ref_pos)
+            else:
+                codon_pos_on_ref.append(self.getPosOnRef(aa_pos, curr_pos))
+        # Get codon seq
+        codon_seq = getStrandedSeqFromPos(
+            self.reference.name,
+            codon_pos_on_ref,
+            self.strand,
+            sequence_reader
+        )
+        return aa_pos, pos_in_codon, codon_seq
