@@ -1,17 +1,93 @@
 # -*- coding: utf-8 -*-
-"""Classes and functions for reading/writing/processing VCF."""
+"""
+Classes and functions for reading/writing/processing VCF.
+
+:Example:
+
+    Read VCF by line
+
+    .. highlight:: python
+    .. code-block:: python
+
+        from anacore.vcf import VCFIO
+
+        with VCFIO("test.vcf.gz") as reader:
+            print("Variant", "\\t".join(reader.samples), sep="\\t")
+            for record in reader:
+                alt_freq = [str(record.getAF(curr_spl)) for curr_spl in reader.samples]
+                print(record.getName(), "\\t".join(alt_freq), sep="\\t")
+
+        # Result>
+        # Variant\tN01\tN02\tN03
+        # chr1:35-35=A/T\t0.3\t0.4\t0.1
+        # chr1:128-128=G/A\t0.1\t0.6\t0.05
+
+    Read VCF by coordinate
+
+    .. highlight:: python
+    .. code-block:: python
+
+        from anacore.vcf import VCFIO
+
+        with VCFIO("test.vcf.gz", "i") as reader:
+            for record in reader.getSub("chr1", 10, 100):
+                print(record.getName())
+
+        # Result>
+        # chr1:35-35=A/T
+
+    Write VCF
+
+    .. highlight:: python
+    .. code-block:: python
+
+        from anacore.vcf import VCFIO
+
+        with VCFIO("test.vcf.gz", "w") as writer:
+            # Header
+            writer.samples = ["my_sample"]
+            writer.filter = [
+                {"q10": HeaderFilterAttr("q10", "Quality below 10")}
+            ]
+            self.info = {
+                "DB": HeaderInfoAttr("DB", "dbSNP membership, build 129", "Flag", 0)
+            }
+            self.format = {
+                "AF": HeaderFormatAttr("AF", "Allele Frequency", "Float", "A")
+            }
+            writer.extra_header = [
+                "##source=myImputationProgramV3.1",
+                "##phasing=partial"
+            ]
+            writer.writeHeader()
+            # Record
+            for record in vcf_record_list:
+                writer.write(record)
+
+        # Result>
+        # ##fileformat=VCFv4.3
+        # ##source=myImputationProgramV3.1
+        # ##phasing=partial
+        # ##INFO=<ID=DB,Number=0,Type=Flag,Description="dbSNP membership, build 129">
+        # ##FILTER=<ID=q10,Description="Quality below 10">
+        # ##FORMAT=<ID=AF,Number=A,Type=Float,Description="Allele Frequency">
+        # #CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\tmy_sample
+        # chr1\t35\t.\tA\tT.\tPASS\tDB\tAF\t0.1
+        # ...
+"""
 
 __author__ = 'Frederic Escudie'
 __copyright__ = 'Copyright (C) 2017 IUCT-O'
 __license__ = 'GNU General Public License'
-__version__ = '1.28.0'
+__version__ = '1.31.0'
 __email__ = 'escudie.frederic@iuct-oncopole.fr'
 __status__ = 'prod'
 
+from anacore.abstractFile import AbstractFile
+from copy import deepcopy
+from pysam import TabixFile
 import sys
 import warnings
-from copy import deepcopy
-from anacore.abstractFile import AbstractFile
 
 
 def decodeInfoValue(val):
@@ -55,12 +131,15 @@ def getHeaderAttr(header_line):
     header_content = header_content[1:-1]  # <ID=AD,Version="1"> to ID=AD,Version="1"
     # Get attributes
     attributes = {}
+    attributes_cases = {}
     stack = ""
     opened_quote = False
     for curr_char in header_content:
         if curr_char == "," and not opened_quote:
             key, val = stack.split("=", 1)
             attributes[key.lower()] = val
+            if key != key.capitalize():  # Keep case different of capitalize
+                attributes_cases[key.lower()] = key
             stack = ""
             opened_quote = False
         elif curr_char == '"':
@@ -76,10 +155,14 @@ def getHeaderAttr(header_line):
     if stack != "":
         key, val = stack.split("=", 1)
         attributes[key.lower()] = val
+        if key != key.capitalize():  # Keep case different of capitalize
+            attributes_cases[key.lower()] = key
     # Return
     header_class_name = "Header{}Attr".format(header_category.capitalize())
     header_class = getattr(sys.modules[__name__], header_class_name)
-    return header_class(**attributes)
+    attr = header_class(**attributes)
+    attr.case_by_attr = attributes_cases
+    return attr
 
 
 class HeaderAttr:
@@ -98,6 +181,7 @@ class HeaderAttr:
         self.id = id
         self._required_attr = ["ID"]  # List of required attributes names.
         self._without_quote = {"ID"}  # List of required attributes without quote in str value.
+        self.case_by_attr = {}  # Store case sensitive title for extra attributes if it is different of capitalized attribute name.
 
     def keys(self):
         """
@@ -118,7 +202,7 @@ class HeaderAttr:
         return self.datastore.items()
 
     def __delattr__(self, name):
-        if name[0] != "_":
+        if name[0] != "_" and name != "case_by_attr":
             del self.datastore[name]
         else:
             super().delattr(name)
@@ -132,14 +216,14 @@ class HeaderAttr:
         :param value: The value to be assigned to the attribute.
         :type value: *
         """
-        if name[0] != "_":
+        if name[0] != "_" and name != "case_by_attr":
             self.datastore[name] = value
         else:
             super().__setattr__(name, value)
 
     def __getattr__(self, name):
         value = None
-        if name[0] != "_":
+        if name[0] != "_" and name != "case_by_attr":
             value = self.datastore[name]
         else:
             value = super().__getattr__(name)
@@ -155,7 +239,10 @@ class HeaderAttr:
         extra_keys = {elt.lower() for elt in self.keys()} - {elt.lower() for elt in self._required_attr}
         if len(extra_keys) != 0:
             for key in sorted(extra_keys):
-                pre_attr.append('{}="{}"'.format(key.capitalize(), self.datastore[key].replace('"', '\\"')))
+                title = key.capitalize()
+                if key in self.case_by_attr:
+                    title = self.case_by_attr[key]
+                pre_attr.append('{}="{}"'.format(title, self.datastore[key].replace('"', '\\"')))
         return "<{}>".format(",".join(pre_attr))
 
     def __repr__(self):
@@ -259,6 +346,12 @@ class HeaderInfoAttr(HeaderTypedAttr):
     pass
 
 
+class HeaderSampleAttr(HeaderAttr):
+    """Class to manage an SAMPLE attribute of VCF header."""
+
+    pass
+
+
 class VCFRecord:
     """Class to manage a variant record."""
 
@@ -377,7 +470,7 @@ class VCFRecord:
                 chr1  10  AAAT  TG => returns 10
         """
         if len(self.alt) > 1:
-            raise Exception("The function 'isDeletion' cannot be used on multi-allelic variant {}.".format(self.getName()))
+            raise Exception("The function 'refStart' cannot be used on multi-allelic variant {}.".format(self.getName()))
         if self._normalized is None:
             self._normalized = deepcopy(self)
             self._normalized.normalizeSingleAllele()
@@ -414,7 +507,7 @@ class VCFRecord:
                 chr1  10  AAAT  TG => returns 13
         """
         if len(self.alt) > 1:
-            raise Exception("The function 'isDeletion' cannot be used on multi-allelic variant {}.".format(self.getName()))
+            raise Exception("The function 'refEnd' cannot be used on multi-allelic variant {}.".format(self.getName()))
         if self._normalized is None:
             self._normalized = deepcopy(self)
             self._normalized.normalizeSingleAllele()
@@ -458,6 +551,33 @@ class VCFRecord:
             is_deletion = True
         return is_deletion
 
+    def isInsAndDel(self):
+        """
+        Return True if the variant is an insertion and also a deletion.
+
+        :return: True if the variant is an insertion and also a deletion.
+        :rtype: bool
+        :warnings: This method can only be used on record with only one alternative allele.
+        """
+        if len(self.alt) > 1:
+            raise Exception("The function 'isInsAndDel' cannot be used on multi-allelic variant {}.".format(self.getName()))
+        is_ins_and_del = False
+        if self._normalized is None:
+            self._normalized = deepcopy(self)
+            self._normalized.normalizeSingleAllele()
+        ref = self._normalized.ref.replace(VCFRecord.getEmptyAlleleMarker(), "")
+        alt = self._normalized.alt[0].replace(VCFRecord.getEmptyAlleleMarker(), "")
+        if len(ref) > len(alt):  # Deletion exists and is more longer (eg: AT/ or AT/G)
+            if len(alt) != 0:  # Insertion exists (eg: AT/G)
+                is_ins_and_del = True
+        elif len(ref) < len(alt):  # Insertion exists and is more longer (eg: /AT or G/AT)
+            if len(ref) != 0:  # Deletion exists (eg: G/AT)
+                is_ins_and_del = True
+        else:  # Same length: SNV or MNV
+            if len(ref) != 1:  # MNV: AT/GC
+                is_ins_and_del = True
+        return is_ins_and_del
+
     def isIndel(self):
         """
         Return True if the variant is an insertion or a deletion.
@@ -509,6 +629,43 @@ class VCFRecord:
             record_type = "variation"
         return record_type
 
+    def fastDownstreamed(self, seq_handler, padding=500):
+        """
+        Return a simplified record (CHROM, POS, ALT and REF) moved to the most downstream postition.
+
+        :param seq_handler: File handle to the reference sequences file.
+        :type seq_handler: anacore.sequenceIO.IdxFastaIO
+        :param padding: Number of nucleotids to inspect after variant. Upstream movement is limited to this number of nucleotids.
+        :type padding: int
+        :return: The simplified record moved to the most downstream postition.
+        :rtype: anacore.vcf.VCFRecord
+        """
+        if len(self.alt) > 1:
+            raise Exception('The function "fastDownstreamed" cannot be used on multi-allelic variant {}.'.format(self.getName()))
+        downstream_rec = VCFRecord(self.chrom, self.pos, None, self.ref, [elt for elt in self.alt])
+        if self._normalized is not None:
+            downstream_rec._normalized = deepcopy(self._normalized)
+        ref = self.ref.replace(VCFRecord.getEmptyAlleleMarker(), "")
+        alt = self.alt[0].replace(VCFRecord.getEmptyAlleleMarker(), "")
+        if len(ref) != 1 or len(alt) != 1:
+            # Move to downstream
+            sub_region = seq_handler.getSub(self.chrom, self.pos, self.pos + len(self.ref) + padding)
+            downstream_rec.pos = 1  # Switch position from chromosome to position from subregion
+            downstream_rec = downstream_rec.getMostDownstream(sub_region)
+            downstream_rec.pos = self.pos + downstream_rec.pos - 1  # Switch position from subregion to position from chromosome
+            if len(ref) != len(alt):
+                # Add previous nt
+                prev_nt = seq_handler.getSub(downstream_rec.chrom, downstream_rec.pos - 1, downstream_rec.pos - 1)
+                if downstream_rec.ref == downstream_rec.getEmptyAlleleMarker():  # Insertion
+                    downstream_rec.ref = prev_nt
+                    downstream_rec.alt[0] = prev_nt + downstream_rec.alt[0]
+                    downstream_rec.pos -= 1
+                elif downstream_rec.alt[0] == VCFRecord.getEmptyAlleleMarker():  # Deletion
+                    downstream_rec.ref = prev_nt + downstream_rec.ref
+                    downstream_rec.alt[0] = prev_nt
+                    downstream_rec.pos -= 1
+        return downstream_rec
+
     def fastStandardize(self, seq_handler, padding=500):
         """
         Standardize record (move to the most upstream position and remove unecessary nucleotids). This standardization concerns only POS, REF and ALT.
@@ -1063,19 +1220,59 @@ class VCFIO(AbstractFile):
 
         :param filepath: The filepath.
         :type filepath: str
-        :param mode: Mode to open the file ('r', 'w', 'a').
+        :param mode: Mode to open the file ('r', 'w', 'a', 'i'). The mode 'i' allow to open file in indexed mode to fetch by region (tabix).
         :type mode: str
         :return: The new instance
         :rtype: anacore.vcf.VCFIO
         """
-        AbstractFile.__init__(self, filepath, mode)
+        standard_mode = mode if mode != "i" else "r"
+        AbstractFile.__init__(self, filepath, standard_mode)
         self.samples = list()
         self.filter = dict()  # {"q10": <HeaderFilterAttr>}
         self.info = dict()  # {"IDREP": <HeaderInfoAttr>, "AD": <HeaderInfoAttr>}
         self.format = dict()  # {"GT": <HeaderFormatAttr>, "AD": <HeaderFormatAttr>}
+        self.sample_info = dict()  # {"splA": <HeaderSampleAttr>, "splB": <HeaderSampleAttr>}
         self.extra_header = list()  # ["##source=myImputationProgramV3.1", "##phasing=partial"]
-        if mode == "r":
+        self._index = None
+        if mode == "a":  # Load header in append mode
+            with VCFIO(filepath, "r") as tmp_reader:
+                self.copyHeader(tmp_reader)
+        if mode in {"r", "i"}:
             self._parseHeader()
+            if mode == "i":
+                self._index = TabixFile(filepath)
+
+    def getSub(self, chr, start, end):
+        """
+        Return generator on records overlapping the specified region.
+
+        .. warning::
+            This method can only be used with an instance of anacore.VCFIO opened
+            with mode "i".
+
+        :param chr: Chromosome name of the selected region.
+        :type chr: str
+        :param start: Start of the selected region (1-based).
+        :type start: int
+        :param end: End of the selected region (1-based).
+        :type end: int
+        :return: Records overlappind the specified region.
+        :rtype: generator for anacore.vcf.VCFRecord
+        """
+        if self._index is None:
+            raise Exception("The file {} must be open in mode 'i' to use {}.getSub() on it.".format(self.file_handle, __class__.__name__))
+        iterator = None
+        try:
+            iterator = self._index.fetch(chr, start - 1, end)
+        except ValueError as error:
+            missing_region = "could not create iterator for region '{}:{}-{}'".format(chr, start, end)
+            if missing_region == str(error):
+                return []
+            else:
+                raise error
+        for row in iterator:
+            self.current_line = row
+            yield self._parseLine()
 
     def isRecordLine(self, line):
         """
@@ -1117,6 +1314,9 @@ class VCFIO(AbstractFile):
         elif self.current_line.startswith("##FORMAT"):
             curr_attr = getHeaderAttr(self.current_line)
             self.format[curr_attr.id] = curr_attr
+        elif self.current_line.startswith("##SAMPLE"):
+            curr_attr = getHeaderAttr(self.current_line)
+            self.sample_info[curr_attr.id] = curr_attr
         elif self.current_line.startswith("#CHROM\tPOS"):
             self.samples = [spl.strip() for spl in self.current_line.split("\t")[9:]]
         elif not self.current_line.startswith("##fileformat="):
@@ -1296,12 +1496,13 @@ class VCFIO(AbstractFile):
         self.filter = deepcopy(model.filter)
         self.format = deepcopy(model.format)
         self.info = deepcopy(model.info)
+        self.sample_info = deepcopy(model.sample_info)
         self.samples = deepcopy(model.samples)
         self.extra_header = deepcopy(model.extra_header)
 
     def writeHeader(self):
         """Write VCF header."""
-        self.file_handle.write("##fileformat=VCFv4.1\n")
+        self.file_handle.write("##fileformat=VCFv4.3\n")
         for curr in self.extra_header:
             self.file_handle.write(curr + "\n")
         for tag in sorted(self.info):
@@ -1310,6 +1511,8 @@ class VCFIO(AbstractFile):
             self.file_handle.write('##FILTER={}\n'.format(self.filter[tag]))
         for tag in sorted(self.format):
             self.file_handle.write('##FORMAT={}\n'.format(self.format[tag]))
+        for spl in sorted(self.sample_info):
+            self.file_handle.write('##SAMPLE={}\n'.format(self.sample_info[spl]))
         last_header_line = "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO"
         if len(self.format) != 0 or len(self.samples) != 0:
             last_header_line += "\tFORMAT\t" + "\t".join([spl for spl in self.samples])
@@ -1320,12 +1523,14 @@ def getAlleleRecord(FH_vcf, record, idx_alt):
     """
     Return the record corresponding to the specified allele in variant.
 
+    :param FH_vcf: Handler on record source file.
+    :type FH_vcf: anacore.vcf.VCFIO
     :param record: The variant record.
-    :type record: VCFRecord
+    :type record: anacore.vcf.VCFRecord
     :param idx_alt: The index of the allele in alt attribute.
     :type idx_alt: int
     :return: The record corresponding to the specified allele in variant.
-    :rtype: VCFRecord
+    :rtype: anacore.vcf.VCFRecord
     """
     new_record = VCFRecord(
         region=record.chrom,
